/*
 * JBoss, Home of Professional Open Source
 * Copyright 2011, Red Hat, Inc. and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jdiameter.common.impl.concurrent;

import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

class BaseThreadFactory implements ThreadFactory {

  public static final String ENTITY_NAME = "ThreadGroup";

  private ThreadGroup threadGroup;
  private String threadPoolName;
  private AtomicInteger count = new AtomicInteger(0);
  // ammendonca: factory should not have this kind of limitations, it's not a pool
  // private int maxCount;

  public BaseThreadFactory(String threadPoolName) {
    this.threadPoolName = threadPoolName;
    // this.maxCount = Integer.MAX_VALUE;
    this.threadGroup = new ThreadGroup("jd " + threadPoolName + " group");
  }

  public BaseThreadFactory(String threadPoolName, int maxCount) {
    this(threadPoolName);
    // this.maxCount = maxCount;
  }

  public Thread newThread(Runnable runnable) {
    // if (count.get() > maxCount) {
    //   throw new RuntimeException("Can not create thread (count=" + count.get() + ", max=" + maxCount + ")");
    // }
    return new Thread(threadGroup, runnable, threadPoolName + "-" + count.getAndIncrement());
  }

  public Thread newThread(String namePrefix, Runnable runnable) {
    // if (count.get() > maxCount) {
    //   throw new RuntimeException("Can not create thread (count=" + count.get() + ", max=" + maxCount + ")");
    // }
    return new Thread(threadGroup, runnable, namePrefix + "-" + count.getAndIncrement());
  }

  public ThreadGroup getThreadGroup() {
    return threadGroup;
  }

}
